/*! @heap-code/object-builder v0.1.2 | License: MIT | Page: https://github.com/heap-code/object-builder | Date: 2024-02-06 */
(function webpackUniversalModuleDefinition(root, factory) {
	if(typeof exports === 'object' && typeof module === 'object')
		module.exports = factory();
	else if(typeof define === 'function' && define.amd)
		define([], factory);
	else {
		var a = factory();
		for(var i in a) (typeof exports === 'object' ? exports : root)[i] = a[i];
	}
})(self, () => {
return /******/ (() => { // webpackBootstrap
/******/ 	"use strict";
/******/ 	// The require scope
/******/ 	var __webpack_require__ = {};
/******/ 	
/************************************************************************/
/******/ 	/* webpack/runtime/define property getters */
/******/ 	(() => {
/******/ 		// define getter functions for harmony exports
/******/ 		__webpack_require__.d = (exports, definition) => {
/******/ 			for(var key in definition) {
/******/ 				if(__webpack_require__.o(definition, key) && !__webpack_require__.o(exports, key)) {
/******/ 					Object.defineProperty(exports, key, { enumerable: true, get: definition[key] });
/******/ 				}
/******/ 			}
/******/ 		};
/******/ 	})();
/******/ 	
/******/ 	/* webpack/runtime/hasOwnProperty shorthand */
/******/ 	(() => {
/******/ 		__webpack_require__.o = (obj, prop) => (Object.prototype.hasOwnProperty.call(obj, prop))
/******/ 	})();
/******/ 	
/******/ 	/* webpack/runtime/make namespace object */
/******/ 	(() => {
/******/ 		// define __esModule on exports
/******/ 		__webpack_require__.r = (exports) => {
/******/ 			if(typeof Symbol !== 'undefined' && Symbol.toStringTag) {
/******/ 				Object.defineProperty(exports, Symbol.toStringTag, { value: 'Module' });
/******/ 			}
/******/ 			Object.defineProperty(exports, '__esModule', { value: true });
/******/ 		};
/******/ 	})();
/******/ 	
/************************************************************************/
var __webpack_exports__ = {};
// ESM COMPAT FLAG
__webpack_require__.r(__webpack_exports__);

// EXPORTS
__webpack_require__.d(__webpack_exports__, {
  ObjectBuilder: () => (/* reexport */ ObjectBuilder),
  ObjectBuilderException: () => (/* reexport */ ObjectBuilderException),
  OverrideUnsetKeyException: () => (/* reexport */ OverrideUnsetKeyException)
});

;// CONCATENATED MODULE: ./src/exceptions/object-builder.exception.ts
class ObjectBuilderException extends Error {
    constructor(...props) {
        super(...props);
        Object.setPrototypeOf(this, new.target.prototype);
    }
}

;// CONCATENATED MODULE: ./src/exceptions/override-unset-key.exception.ts

class OverrideUnsetKeyException extends ObjectBuilderException {
    constructor(key) {
        super(`The key '${key.toString()}' must already be set to be overridden.`);
    }
}

;// CONCATENATED MODULE: ./src/exceptions/index.ts



;// CONCATENATED MODULE: ./src/object-builder.ts

class ObjectBuilder {
    static create() {
        return new ObjectBuilder(new Map());
    }
    constructor(handlers) {
        this.handlers = handlers;
    }
    build() {
        return Array.from(this.handlers.entries()).reduce((self, [key, handler]) => Object.defineProperty(self, key, {
            get: () => handler(self),
        }), {});
    }
    keys() {
        return Array.from(this.handlers.keys());
    }
    with(key, handler) {
        return new ObjectBuilder(new Map([...Array.from(this.handlers.entries()), [key, handler]]));
    }
    override(key, handler) {
        const previous = this.handlers.get(key);
        if (!previous) {
            throw new OverrideUnsetKeyException(key);
        }
        return this.with(key, self => handler(self, previous(self)));
    }
}

;// CONCATENATED MODULE: ./src/index.ts




/******/ 	return __webpack_exports__;
/******/ })()
;
});
//# sourceMappingURL=object-builder.umd.js.map