"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ObjectBuilder = void 0;
const exceptions_1 = require("./exceptions");
class ObjectBuilder {
    static create() {
        return new ObjectBuilder(new Map());
    }
    constructor(handlers) {
        this.handlers = handlers;
    }
    build() {
        return Array.from(this.handlers.entries()).reduce((self, [key, handler]) => Object.defineProperty(self, key, {
            get: () => handler(self),
        }), {});
    }
    keys() {
        return Array.from(this.handlers.keys());
    }
    with(key, handler) {
        return new ObjectBuilder(new Map([...Array.from(this.handlers.entries()), [key, handler]]));
    }
    override(key, handler) {
        const previous = this.handlers.get(key);
        if (!previous) {
            throw new exceptions_1.OverrideUnsetKeyException(key);
        }
        return this.with(key, self => handler(self, previous(self)));
    }
}
exports.ObjectBuilder = ObjectBuilder;
//# sourceMappingURL=object-builder.js.map